import { DataGrid, GridColDef, GridRenderCellParams } from "@mui/x-data-grid";
import { ruRU } from "@mui/x-data-grid/locales";
import { articlesStore, languageStore } from "@shared";
import { useEffect, useState } from "react";
import { observer } from "mobx-react-lite";
import { Trash2, Eye, Minus } from "lucide-react";
import { useNavigate } from "react-router-dom";
import { DeleteModal, LanguageSwitcher } from "@widgets";
import { Box, CircularProgress } from "@mui/material";

export const ArticleListPage = observer(() => {
  const { articleList, getArticleList, deleteArticles } = articlesStore;
  const navigate = useNavigate();
  const [isDeleteModalOpen, setIsDeleteModalOpen] = useState(false);
  const [isBulkDeleteModalOpen, setIsBulkDeleteModalOpen] = useState(false);
  const [rowId, setRowId] = useState<string | null>(null);
  const { language } = languageStore;
  const [ids, setIds] = useState<number[]>([]);
  const [isLoading, setIsLoading] = useState(false);

  useEffect(() => {
    const fetchArticles = async () => {
      setIsLoading(true);
      await getArticleList();
      setIsLoading(false);
    };
    fetchArticles();
  }, [language]);

  const columns: GridColDef[] = [
    {
      field: "heading",
      headerName: "Название",
      flex: 1,
      renderCell: (params: GridRenderCellParams) => {
        return params.value ? (
          params.value
        ) : (
          <div className="flex h-full gap-7 items-center">
            <Minus size={20} className="text-red-500" />
          </div>
        );
      },
    },

    {
      field: "actions",
      headerName: "Действия",

      renderCell: (params: GridRenderCellParams) => {
        return (
          <div className="flex h-full gap-7 justify-center items-center">
            <button onClick={() => navigate(`/article/${params.row.id}`)}>
              <Eye size={20} className="text-green-500" />
            </button>
            <button
              onClick={() => {
                setIsDeleteModalOpen(true);
                setRowId(params.row.id);
              }}
            >
              <Trash2 size={20} className="text-red-500" />
            </button>
          </div>
        );
      },
    },
  ];

  const rows = articleList[language].data.map((article) => ({
    id: article.id,
    heading: article.heading,
    body: article.body,
  }));

  return (
    <>
      <LanguageSwitcher />

      <div className="w-full">
        <div className="flex justify-between items-center mb-10">
          <h1 className="text-2xl">Статьи</h1>
        </div>

        <div
          className="flex justify-end mb-5 duration-300"
          style={{ opacity: ids.length > 0 ? 1 : 0 }}
        >
          <button
            className="px-4 py-2 bg-red-500 text-white rounded flex gap-2 items-center"
            onClick={() => setIsBulkDeleteModalOpen(true)}
          >
            <Trash2 size={20} className="text-white" /> Удалить выбранные (
            {ids.length})
          </button>
        </div>

        <div className="w-full">
          <DataGrid
            rows={rows}
            columns={columns}
            hideFooterPagination
            checkboxSelection
            loading={isLoading}
            localeText={ruRU.components.MuiDataGrid.defaultProps.localeText}
            onRowSelectionModelChange={(newSelection) => {
              setIds(Array.from(newSelection.ids) as number[]);
            }}
            hideFooter
            slots={{
              noRowsOverlay: () => (
                <Box
                  sx={{ mt: 5, textAlign: "center", color: "text.secondary" }}
                >
                  {isLoading ? <CircularProgress size={20} /> : "Нет статей"}
                </Box>
              ),
            }}
          />
        </div>
      </div>

      <DeleteModal
        open={isDeleteModalOpen}
        onDelete={async () => {
          if (rowId) {
            await deleteArticles([parseInt(rowId)]);
            getArticleList();
          }
          setIsDeleteModalOpen(false);
          setRowId(null);
        }}
        onCancel={() => {
          setIsDeleteModalOpen(false);
          setRowId(null);
        }}
      />

      <DeleteModal
        open={isBulkDeleteModalOpen}
        onDelete={async () => {
          await deleteArticles(ids);
          getArticleList();
          setIsBulkDeleteModalOpen(false);
          setIds([]);
        }}
        onCancel={() => {
          setIsBulkDeleteModalOpen(false);
        }}
      />
    </>
  );
});
