import { useState, useEffect } from "react";
import {
  Stack,
  Typography,
  Button,
  FormControl,
  Accordion,
  AccordionSummary,
  AccordionDetails,
  useTheme,
  TextField,
  Autocomplete,
  TableCell,
  TableContainer,
  Table,
  TableHead,
  TableRow,
  Paper,
  TableBody,
  IconButton,
  Checkbox,
  FormControlLabel,
  Tabs,
  Tab,
  Box,
} from "@mui/material";
import ExpandMoreIcon from "@mui/icons-material/ExpandMore";
import DragIndicatorIcon from "@mui/icons-material/DragIndicator";
import {
  DragDropContext,
  Droppable,
  Draggable,
  DropResult,
} from "@hello-pangea/dnd";

import { authInstance, languageStore, routeStore } from "@shared";
import { EditStationModal } from "../../widgets/modals/EditStationModal";

// Helper function to insert an item at a specific position (1-based index)
function insertAtPosition<T>(arr: T[], pos: number, value: T): T[] {
  const index = pos - 1;
  const result = [...arr];
  if (index >= result.length) {
    result.push(value);
  } else {
    result.splice(index, 0, value);
  }
  return result;
}

// Helper function to reorder items after drag and drop
const reorder = <T,>(list: T[], startIndex: number, endIndex: number): T[] => {
  const result = Array.from(list);
  const [removed] = result.splice(startIndex, 1);
  result.splice(endIndex, 0, removed);
  return result;
};

type Field<T> = {
  label: string;
  data: keyof T;
  render?: (value: any) => React.ReactNode;
};

type LinkedItemsProps<T> = {
  parentId: string | number;
  fields: Field<T>[];
  setItemsParent?: (items: T[]) => void;
  type: "show" | "edit";
  dragAllowed?: boolean;
  onUpdate?: () => void;
  dontRecurse?: boolean;
  disableCreation?: boolean;
  updatedLinkedItems?: T[];
  refresh?: number;
  cityId?: number;
  routeDirection?: boolean;
};

export const LinkedItems = <
  T extends { id: number; name: string; [key: string]: any }
>(
  props: LinkedItemsProps<T>
) => {
  const theme = useTheme();

  return (
    <>
      <Accordion sx={{ width: "100%" }}>
        <AccordionSummary
          expandIcon={<ExpandMoreIcon />}
          sx={{
            background: theme.palette.background.paper,
            borderBottom: `1px solid ${theme.palette.divider}`,
            width: "100%",
          }}
        >
          <Typography variant="subtitle1" fontWeight="bold">
            Привязанные станции
          </Typography>
        </AccordionSummary>

        <AccordionDetails
          sx={{ background: theme.palette.background.paper, width: "100%" }}
        >
          <Stack gap={2} width="100%">
            <LinkedItemsContents {...props} />
          </Stack>
        </AccordionDetails>
      </Accordion>
    </>
  );
};

export const LinkedItemsContents = <
  T extends { id: number; name: string; [key: string]: any }
>({
  parentId,
  setItemsParent,
  fields,
  dragAllowed = false,
  type,
  onUpdate,
  disableCreation = false,
  updatedLinkedItems,
  refresh,
  cityId,
  routeDirection,
}: LinkedItemsProps<T>) => {
  const { language } = languageStore;

  const [position, setPosition] = useState<number>(1);
  const [allItems, setAllItems] = useState<T[]>([]);
  const [linkedItems, setLinkedItems] = useState<T[]>([]);
  const [selectedItemId, setSelectedItemId] = useState<number | null>(null);
  const [isLoading, setIsLoading] = useState<boolean>(true);
  const [error, setError] = useState<string | null>(null);
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [selectedItems, setSelectedItems] = useState<Set<number>>(new Set());
  const [activeTab, setActiveTab] = useState(0);
  const [searchQuery, setSearchQuery] = useState("");

  useEffect(() => {}, [error]);

  const parentResource = "route";
  const childResource = "station";

  const availableItems = allItems
    .filter((item) => !linkedItems.some((linked) => linked.id === item.id))
    .filter((item) => {
      // Если направление маршрута не указано, показываем все станции
      if (routeDirection === undefined) return true;
      // Фильтруем станции по направлению маршрута
      return item.direction === routeDirection;
    })
    .sort((a, b) => a.name.localeCompare(b.name));

  // Фильтрация по поиску для массового режима
  const filteredAvailableItems = availableItems.filter((item) => {
    if (!cityId || item.city_id == cityId) {
      if (!searchQuery.trim()) return true;
      return String(item.name)
        .toLowerCase()
        .includes(searchQuery.toLowerCase());
    }
    return false;
  });

  useEffect(() => {
    if (updatedLinkedItems) {
      setLinkedItems(updatedLinkedItems);
    }
  }, [updatedLinkedItems]);

  useEffect(() => {
    setItemsParent?.(linkedItems);
  }, [linkedItems, setItemsParent]);

  useEffect(() => {
    setPosition(linkedItems.length + 1);
  }, [linkedItems.length]);

  const onDragEnd = (result: DropResult) => {
    if (!result.destination) return;

    const reorderedItems = reorder(
      linkedItems,
      result.source.index,
      result.destination.index
    );

    setLinkedItems(reorderedItems);

    authInstance
      .post(`/${parentResource}/${parentId}/${childResource}`, {
        stations: reorderedItems.map((item) => ({ id: item.id })),
      })
      .catch((error) => {
        console.error("Error updating station order:", error);
        setError("Failed to update station order");
      });
  };

  useEffect(() => {
    if (parentId) {
      setIsLoading(true);
      setError(null);
      authInstance
        .get(`/${parentResource}/${parentId}/${childResource}`)
        .then((response) => {
          setLinkedItems(response?.data || []);
        })
        .catch((error) => {
          console.error("Error fetching linked items:", error);
          setError("Failed to load linked stations");
          setLinkedItems([]);
        })
        .finally(() => {
          setIsLoading(false);
        });
    }
  }, [parentId, language, refresh]);

  useEffect(() => {
    if (type === "edit") {
      setError(null);
      authInstance
        .get(`/${childResource}/`)
        .then((response) => {
          setAllItems(response?.data || []);
        })
        .catch((error) => {
          console.error("Error fetching all items:", error);
          setError("Failed to load available stations");
          setAllItems([]);
        });
    }
  }, [type]);

  const linkItem = () => {
    if (selectedItemId !== null) {
      setError(null);
      const requestData = {
        stations: insertAtPosition(
          linkedItems.map((item) => ({ id: item.id })),
          position,
          { id: selectedItemId }
        ),
      };

      authInstance
        .post(`/${parentResource}/${parentId}/${childResource}`, requestData)
        .then(() => {
          const newItem = allItems.find((item) => item.id === selectedItemId);
          if (newItem) {
            const updatedList = insertAtPosition(
              [...linkedItems],
              position,
              newItem
            );
            setLinkedItems(updatedList);
          }
          setSelectedItemId(null);
          onUpdate?.();
        })
        .catch((error) => {
          console.error("Error linking item:", error);
          setError("Failed to link station");
        });
    }
  };

  const deleteItem = (itemId: number) => {
    setError(null);
    authInstance
      .delete(`/${parentResource}/${parentId}/${childResource}`, {
        data: { [`${childResource}_id`]: itemId },
      })
      .then(() => {
        setLinkedItems(linkedItems.filter((item) => item.id !== itemId));
        onUpdate?.();
      })
      .catch((error) => {
        console.error("Error deleting item:", error);
        setError("Failed to delete station");
      });
  };

  const handleStationClick = (item: T) => {
    routeStore.setSelectedStationId(item.id);
    setIsModalOpen(true);
  };

  const handleCloseModal = () => {
    setIsModalOpen(false);
  };

  const handleCheckboxChange = (itemId: number) => {
    const newSelected = new Set(selectedItems);
    if (newSelected.has(itemId)) {
      newSelected.delete(itemId);
    } else {
      newSelected.add(itemId);
    }
    setSelectedItems(newSelected);
  };

  const handleBulkLink = () => {
    if (selectedItems.size === 0) return;

    setError(null);
    const selectedStations = Array.from(selectedItems).map((id) => ({ id }));
    const requestData = {
      stations: [
        ...linkedItems.map((item) => ({ id: item.id })),
        ...selectedStations,
      ],
    };

    authInstance
      .post(`/${parentResource}/${parentId}/${childResource}`, requestData)
      .then(() => {
        const newItems = allItems.filter((item) => selectedItems.has(item.id));
        setLinkedItems([...linkedItems, ...newItems]);
        setSelectedItems(new Set());
        onUpdate?.();
      })
      .catch((error) => {
        console.error("Error linking items:", error);
        setError("Failed to link stations");
      });
  };

  return (
    <>
      {linkedItems?.length > 0 && (
        <DragDropContext onDragEnd={onDragEnd}>
          <TableContainer component={Paper} sx={{ width: "100%" }}>
            <Table sx={{ width: "100%" }}>
              <TableHead>
                <TableRow>
                  {type === "edit" && dragAllowed && (
                    <TableCell width="40px"></TableCell>
                  )}
                  <TableCell key="id" width="60px">
                    №
                  </TableCell>
                  {fields.map((field) => (
                    <TableCell key={String(field.data)}>
                      {field.label}
                    </TableCell>
                  ))}
                  {type === "edit" && (
                    <TableCell width="120px">Действие</TableCell>
                  )}
                </TableRow>
              </TableHead>

              <Droppable
                droppableId="droppable-stations"
                isDropDisabled={type !== "edit" || !dragAllowed}
              >
                {(provided) => (
                  <TableBody
                    ref={provided.innerRef}
                    {...provided.droppableProps}
                  >
                    {linkedItems.map((item, index) => (
                      <Draggable
                        key={item.id}
                        draggableId={"station-" + String(item.id)}
                        index={index}
                        isDragDisabled={type !== "edit" || !dragAllowed}
                      >
                        {(provided) => (
                          <TableRow
                            sx={{ cursor: "pointer" }}
                            ref={provided.innerRef}
                            {...provided.draggableProps}
                            hover
                            onClick={() => handleStationClick(item)}
                          >
                            {type === "edit" && dragAllowed && (
                              <TableCell {...provided.dragHandleProps}>
                                <IconButton size="small">
                                  <DragIndicatorIcon />
                                </IconButton>
                              </TableCell>
                            )}
                            <TableCell>{index + 1}</TableCell>
                            {fields.map((field, idx) => (
                              <TableCell key={String(field.data) + String(idx)}>
                                {field.render
                                  ? field.render(item[field.data])
                                  : item[field.data]}
                              </TableCell>
                            ))}
                            {type === "edit" && (
                              <TableCell>
                                <Button
                                  variant="outlined"
                                  color="error"
                                  size="small"
                                  onClick={(e) => {
                                    e.stopPropagation();
                                    deleteItem(item.id);
                                  }}
                                >
                                  Отвязать
                                </Button>
                              </TableCell>
                            )}
                          </TableRow>
                        )}
                      </Draggable>
                    ))}
                    {provided.placeholder}
                  </TableBody>
                )}
              </Droppable>
            </Table>
          </TableContainer>
        </DragDropContext>
      )}

      {linkedItems.length === 0 && !isLoading && (
        <Typography color="textSecondary" textAlign="center" py={2}>
          Станции не найдены
        </Typography>
      )}

      {type === "edit" && !disableCreation && (
        <Stack gap={2} mt={2}>
          <Typography variant="subtitle1">Добавить остановки</Typography>
          {routeDirection !== undefined && (
            <Typography variant="body2" color="textSecondary">
              Показываются только остановки для{" "}
              {routeDirection ? "прямого" : "обратного"} направления
            </Typography>
          )}

          <Tabs
            value={activeTab}
            onChange={(_, newValue) => setActiveTab(newValue)}
          >
            <Tab label="По одной" />
            <Tab label="Массово" />
          </Tabs>

          <Box sx={{ mt: 2 }}>
            {activeTab === 0 && (
              <Stack gap={2}>
                <Autocomplete
                  fullWidth
                  value={
                    availableItems?.find(
                      (item) => item.id === selectedItemId
                    ) || null
                  }
                  onChange={(_, newValue) =>
                    setSelectedItemId(newValue?.id || null)
                  }
                  options={availableItems.filter(
                    (item) => !cityId || item.city_id == cityId
                  )}
                  getOptionLabel={(item) => String(item.name)}
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      label="Выберите остановку"
                      fullWidth
                    />
                  )}
                  isOptionEqualToValue={(option, value) =>
                    option.id === value?.id
                  }
                  filterOptions={(options, { inputValue }) => {
                    const searchWords = inputValue
                      .toLowerCase()
                      .split(" ")
                      .filter(Boolean);
                    return options.filter((option) => {
                      const optionWords = String(option.name)
                        .toLowerCase()
                        .split(" ");
                      return searchWords.every((searchWord) =>
                        optionWords.some((word) => word.startsWith(searchWord))
                      );
                    });
                  }}
                  renderOption={(props, option) => (
                    <li {...props} key={option.id}>
                      <div className="flex justify-between items-center w-full">
                        <p>{String(option.name)}</p>
                        <p className="text-xs text-gray-500 max-w-[300px] mr-4 truncate">
                          {String(option.description)}
                        </p>
                      </div>
                    </li>
                  )}
                />

                <FormControl fullWidth>
                  <TextField
                    type="number"
                    label="Позиция добавляемой остановки"
                    value={position}
                    onChange={(e) => {
                      const newValue = Math.max(1, Number(e.target.value));
                      setPosition(
                        newValue > linkedItems.length + 1
                          ? linkedItems.length + 1
                          : newValue
                      );
                    }}
                    InputProps={{
                      inputProps: { min: 1, max: linkedItems.length + 1 },
                    }}
                    fullWidth
                  />
                </FormControl>

                <Button
                  variant="contained"
                  onClick={linkItem}
                  disabled={!selectedItemId}
                  sx={{ alignSelf: "flex-start" }}
                >
                  Добавить
                </Button>
              </Stack>
            )}

            {activeTab === 1 && (
              <Stack gap={2}>
                {/* Поле поиска */}
                <TextField
                  fullWidth
                  label="Поиск остановок"
                  value={searchQuery}
                  onChange={(e) => setSearchQuery(e.target.value)}
                  placeholder="Введите название остановки..."
                  size="small"
                  sx={{ mb: 1 }}
                />

                {/* Список доступных остановок с чекбоксами */}
                <Paper sx={{ maxHeight: 300, overflow: "auto", p: 2 }}>
                  <Stack gap={1}>
                    {filteredAvailableItems.map((item) => (
                      <FormControlLabel
                        key={item.id}
                        control={
                          <Checkbox
                            checked={selectedItems.has(item.id)}
                            onChange={() => handleCheckboxChange(item.id)}
                            size="small"
                          />
                        }
                        label={String(item.name)}
                        sx={{
                          margin: 0,
                          "& .MuiFormControlLabel-label": {
                            fontSize: "0.875rem",
                          },
                        }}
                      />
                    ))}
                    {filteredAvailableItems.length === 0 && (
                      <Typography
                        color="textSecondary"
                        textAlign="center"
                        py={1}
                      >
                        {searchQuery.trim()
                          ? "Остановки не найдены"
                          : "Нет доступных остановок"}
                      </Typography>
                    )}
                  </Stack>
                </Paper>

                <Button
                  variant="contained"
                  onClick={handleBulkLink}
                  disabled={selectedItems.size === 0}
                  sx={{ alignSelf: "flex-start" }}
                >
                  Добавить выбранные ({selectedItems.size})
                </Button>
              </Stack>
            )}
          </Box>
        </Stack>
      )}
      <EditStationModal open={isModalOpen} onClose={handleCloseModal} />
    </>
  );
};
