// @shared/stores/createSightStore.ts
import {
  articlesStore,
  Language,
  authInstance,
  languageInstance,
  mediaStore,
} from "@shared";
import { makeAutoObservable, runInAction } from "mobx";

type MediaItem = {
  id: string;
  filename: string;
  media_name?: string;
  media_type: number;
};

type SightLanguageInfo = {
  name: string;
  address: string;
  left: {
    heading: string;
    body: string;
    media: MediaItem[];
  };
  right: { id: number; heading: string; body: string; media: MediaItem[] }[];
};

type SightCommonInfo = {
  // id: number; // ID is 0 until created
  city_id: number;
  city: string;
  latitude: number;
  longitude: number;
  thumbnail: string | null;
  watermark_lu: string | null;
  watermark_rd: string | null;
  left_article: number; // Can be 0 or a real ID, or placeholder like 10000000
  preview_media: string | null;
  video_preview: string | null;
};

// SightBaseInfo combines common info with language-specific info
// The 'id' for the sight itself will be assigned upon creation by the backend.
type SightBaseInfo = SightCommonInfo & {
  [key in Language]: SightLanguageInfo;
};

const initialSightState: SightBaseInfo = {
  city_id: 0,
  city: "",
  latitude: 0,
  longitude: 0,
  thumbnail: null,
  watermark_lu: null,
  watermark_rd: null,
  left_article: 0,
  preview_media: null,
  video_preview: null,
  ru: {
    name: "",
    address: "",
    left: { heading: "", body: "", media: [] },
    right: [],
  },
  en: {
    name: "",
    address: "",
    left: { heading: "", body: "", media: [] },
    right: [],
  },
  zh: {
    name: "",
    address: "",
    left: { heading: "", body: "", media: [] },
    right: [],
  },
};

class CreateSightStore {
  sight: SightBaseInfo = JSON.parse(JSON.stringify(initialSightState)); // Deep copy for reset

  uploadMediaOpen = false;
  setUploadMediaOpen = (open: boolean) => {
    this.uploadMediaOpen = open;
  };
  fileToUpload: File | null = null;
  setFileToUpload = (file: File | null) => {
    this.fileToUpload = file;
  };

  constructor() {
    makeAutoObservable(this);
  }

  // --- Right Article Management ---
  createNewRightArticle = async () => {
    // Create article in DB for all languages
    const articleRuData = {
      heading: "Новый заголовок (RU)",
      body: "Новый текст (RU)",
    };
    const articleEnData = {
      heading: "New Heading (EN)",
      body: "New Text (EN)",
    };
    const articleZhData = {
      heading: "Новый заголовок (ZH)",
      body: "Новый текст (ZH)",
    };

    try {
      this.needLeaveAgree = true;
      const articleRes = await authInstance.post("/article", {
        translations: {
          heading: {
            ru: articleRuData.heading,
            en: articleEnData.heading,
            zh: articleZhData.heading,
          },
          body: {
            ru: articleRuData.body,
            en: articleEnData.body,
            zh: articleZhData.body,
          },
        },
      });
      const { id } = articleRes.data; // New article's ID

      runInAction(() => {
        const newArticleEntry = { id, media: [] };
        this.sight.ru.right.push({ ...newArticleEntry, ...articleRuData });
        this.sight.en.right.push({ ...newArticleEntry, ...articleEnData });
        this.sight.zh.right.push({ ...newArticleEntry, ...articleZhData });
      });
      return id; // Return ID for potential immediate use
    } catch (error) {
      console.error("Error creating new right article:", error);
      throw error;
    }
  };

  linkExistingRightArticle = async (articleId: number) => {
    try {
      const ruData = await languageInstance("ru").get(`/article/${articleId}`);
      const enData = await languageInstance("en").get(`/article/${articleId}`);
      const zhData = await languageInstance("zh").get(`/article/${articleId}`);
      const mediaRes = await authInstance.get(`/article/${articleId}/media`);
      const mediaData: MediaItem[] = mediaRes.data || [];

      runInAction(() => {
        this.sight.ru.right.push({
          id: articleId,
          heading: ruData.data.heading,
          body: ruData.data.body,
          media: mediaData,
        });
        this.sight.en.right.push({
          id: articleId,
          heading: enData.data.heading,
          body: enData.data.body,
          media: mediaData,
        });
        this.sight.zh.right.push({
          id: articleId,
          heading: zhData.data.heading,
          body: zhData.data.body,
          media: mediaData,
        });
      });

      return articleId; // Return the linked article ID
    } catch (error) {
      console.error("Error linking existing right article:", error);
      throw error;
    }
  };

  updateRightArticleInfo = (
    index: number,
    language: Language,
    heading: string,
    body: string
  ) => {
    if (this.sight[language].right[index]) {
      this.sight[language].right[index].heading = heading;
      this.sight[language].right[index].body = body;
    }
  };

  // "Unlink" in create mode means just removing from the list to be created with the sight
  unlinkRightAritcle = (articleId: number) => {
    // Changed from 'unlinkRightAritcle' spelling
    runInAction(() => {
      this.sight.ru.right = this.sight.ru.right.filter(
        (article) => article.id !== articleId
      );
      this.sight.en.right = this.sight.en.right.filter(
        (article) => article.id !== articleId
      );
      this.sight.zh.right = this.sight.zh.right.filter(
        (article) => article.id !== articleId
      );
    });
    // Note: If this article was created via createNewRightArticle, it still exists in the DB.
    // Consider if an orphaned article should be deleted here or managed separately.
    // For now, it just removes it from the list associated with *this specific sight creation process*.
  };

  deleteRightArticle = async (articleId: number) => {
    try {
      await authInstance.delete(`/article/${articleId}`); // Delete from backend
      runInAction(() => {
        // Remove from local store for all languages
        this.sight.ru.right = this.sight.ru.right.filter(
          (article) => article.id !== articleId
        );
        this.sight.en.right = this.sight.en.right.filter(
          (article) => article.id !== articleId
        );
        this.sight.zh.right = this.sight.zh.right.filter(
          (article) => article.id !== articleId
        );
      });
    } catch (error) {
      console.error("Error deleting right article:", error);
      throw error;
    }
  };

  // --- Right Article Media Management ---
  createLinkWithRightArticle = async (media: MediaItem, articleId: number) => {
    try {
      await authInstance.post(`/article/${articleId}/media`, {
        media_id: media.id,
        media_order: 1, // Or calculate based on existing media.length + 1
      });
      runInAction(() => {
        (["ru", "en", "zh"] as Language[]).forEach((lang) => {
          const article = this.sight[lang].right.find(
            (a) => a.id === articleId
          );
          if (article) {
            if (!article.media) article.media = [];
            article.media.unshift(media); // Add to the beginning
          }
        });
      });
    } catch (error) {
      console.error("Error linking media to right article:", error);
      throw error;
    }
  };

  deleteRightArticleMedia = async (articleId: number, mediaId: string) => {
    try {
      await authInstance.delete(`/article/${articleId}/media`, {
        data: { media_id: mediaId },
      });
      runInAction(() => {
        (["ru", "en", "zh"] as Language[]).forEach((lang) => {
          const article = this.sight[lang].right.find(
            (a) => a.id === articleId
          );
          if (article && article.media) {
            article.media = article.media.filter((m) => m.id !== mediaId);
          }
        });
      });
    } catch (error) {
      console.error("Error deleting media from right article:", error);
      throw error;
    }
  };

  // --- Left Article Management (largely unchanged from your provided store) ---
  updateLeftInfo = (language: Language, heading: string, body: string) => {
    this.sight[language].left.heading = heading;
    this.sight[language].left.body = body;
  };

  unlinkLeftArticle = () => {
    /* ... your existing logic ... */
    this.sight.left_article = 0;
    this.sight.ru.left = { heading: "", body: "", media: [] };
    this.sight.en.left = { heading: "", body: "", media: [] };
    this.sight.zh.left = { heading: "", body: "", media: [] };
  };

  updateLeftArticle = async (articleId: number) => {
    /* ... your existing logic ... */
    this.sight.left_article = articleId;
    if (articleId) {
      const [ruArticleData, enArticleData, zhArticleData, mediaData] =
        await Promise.all([
          languageInstance("ru").get(`/article/${articleId}`),
          languageInstance("en").get(`/article/${articleId}`),
          languageInstance("zh").get(`/article/${articleId}`),
          authInstance.get(`/article/${articleId}/media`),
        ]);
      runInAction(() => {
        this.sight.ru.left = {
          heading: ruArticleData.data.heading,
          body: ruArticleData.data.body,
          media: mediaData.data || [],
        };
        this.sight.en.left = {
          heading: enArticleData.data.heading,
          body: enArticleData.data.body,
          media: mediaData.data || [],
        };
        this.sight.zh.left = {
          heading: zhArticleData.data.heading,
          body: zhArticleData.data.body,
          media: mediaData.data || [],
        };
      });
    } else {
      this.unlinkLeftArticle();
    }
  };

  deleteLeftArticle = async (articleId: number) => {
    /* ... your existing logic ... */
    await authInstance.delete(`/article/${articleId}`);
    // articlesStore.getArticles(languageStore.language); // If still neede
    runInAction(() => {
      articlesStore.articles.ru = articlesStore.articles.ru.filter(
        (article) => article.id !== articleId
      );
      articlesStore.articles.en = articlesStore.articles.en.filter(
        (article) => article.id !== articleId
      );
      articlesStore.articles.zh = articlesStore.articles.zh.filter(
        (article) => article.id !== articleId
      );
    });
    this.unlinkLeftArticle();
  };

  createLeftArticle = async () => {
    /* ... your existing logic to create a new left article (placeholder or DB) ... */
    const response = await languageInstance("ru").post("/article", {
      heading: "Новая левая статья",
      body: "Заполните контентом",
    });
    const newLeftArticleId = response.data.id;
    await languageInstance("en").patch(`/article/${newLeftArticleId}`, {
      heading: "New Left Article",
      body: "Fill with content",
    });
    await languageInstance("zh").patch(`/article/${newLeftArticleId}`, {
      heading: "新的左侧文章",
      body: "填写内容",
    });

    runInAction(() => {
      this.sight.left_article = newLeftArticleId; // Store the actual ID
      this.sight.ru.left = {
        heading: "Новая левая статья",
        body: "Заполните контентом",
        media: [],
      };
      this.sight.en.left = {
        heading: "New Left Article",
        body: "Fill with content",
        media: [],
      };
      this.sight.zh.left = {
        heading: "新的左侧文章",
        body: "填写内容",
        media: [],
      };

      articlesStore.articles.ru.push({
        id: newLeftArticleId,
        heading: "Новая левая статья",
        body: "Заполните контентом",
        service_name: "Новая левая статья",
      });
      articlesStore.articles.en.push({
        id: newLeftArticleId,
        heading: "New Left Article",
        body: "Fill with content",
        service_name: "New Left Article",
      });
      articlesStore.articles.zh.push({
        id: newLeftArticleId,
        heading: "新的左侧文章",
        body: "填写内容",
        service_name: "新的左侧文章",
      });
    });
    return newLeftArticleId;
  };

  // Placeholder for a "new" unsaved left article
  setNewLeftArticlePlaceholder = () => {
    this.sight.left_article = 10000000; // Special placeholder ID
    this.sight.ru.left = {
      heading: "Новая левая статья",
      body: "Заполните контентом",
      media: [],
    };
    this.sight.en.left = {
      heading: "New Left Article",
      body: "Fill with content",
      media: [],
    };
    this.sight.zh.left = {
      heading: "新的左侧文章",
      body: "填写内容",
      media: [],
    };
  };

  // --- Sight Preview Media ---
  linkPreviewMedia = (mediaId: string) => {
    this.sight.preview_media = mediaId;
  };

  unlinkPreviewMedia = () => {
    this.sight.preview_media = null;
  };

  // --- General Store Methods ---
  clearCreateSight = () => {
    this.needLeaveAgree = false;
    this.sight = JSON.parse(JSON.stringify(initialSightState)); // Reset to initial
  };

  updateSightInfo = (
    content: Partial<SightLanguageInfo | SightCommonInfo>, // Corrected types
    language?: Language
  ) => {
    this.needLeaveAgree = true;
    if (language) {
      this.sight[language] = { ...this.sight[language], ...content };
    } else {
      // Assuming content here is for SightCommonInfo
      this.sight = { ...this.sight, ...(content as Partial<SightCommonInfo>) };
    }
  };

  // --- Main Sight Creation Logic ---
  createSight = async (primaryLanguage: Language) => {
    let finalLeftArticleId = this.sight.left_article;

    // 1. Handle Left Article (Create if new, or use existing ID)
    if (this.sight.left_article === 10000000) {
      // Placeholder for new
      const res = await languageInstance("ru").post("/article", {
        heading: this.sight.ru.left.heading,
        body: this.sight.ru.left.body,
      });
      finalLeftArticleId = res.data.id;
      await languageInstance("en").patch(`/article/${finalLeftArticleId}`, {
        heading: this.sight.en.left.heading,
        body: this.sight.en.left.body,
      });
      await languageInstance("zh").patch(`/article/${finalLeftArticleId}`, {
        heading: this.sight.zh.left.heading,
        body: this.sight.zh.left.body,
      });
    } else if (
      this.sight.left_article !== 0 &&
      this.sight.left_article !== null
    ) {
      // Existing, ensure it's up-to-date
      await languageInstance("ru").patch(
        `/article/${this.sight.left_article}`,
        { heading: this.sight.ru.left.heading, body: this.sight.ru.left.body }
      );
      await languageInstance("en").patch(
        `/article/${this.sight.left_article}`,
        { heading: this.sight.en.left.heading, body: this.sight.en.left.body }
      );
      await languageInstance("zh").patch(
        `/article/${this.sight.left_article}`,
        { heading: this.sight.zh.left.heading, body: this.sight.zh.left.body }
      );
    }
    // else: left_article is 0, so no left article

    // 2. Right articles are already created in DB and their IDs are in this.sight[lang].right.
    // We just need to update their content if changed before saving the sight.
    for (const lang of ["ru", "en", "zh"] as Language[]) {
      for (const article of this.sight[lang].right) {
        if (article.id == 0 || article.id == null) {
          continue;
        }
        await languageInstance(lang).patch(`/article/${article.id}`, {
          heading: article.heading,
          body: article.body,
        });
        // Media for these articles are already linked via createLinkWithRightArticle
      }
    }
    const rightArticleIdsForLink = this.sight[primaryLanguage].right.map(
      (a) => a.id
    );

    // 3. Create Sight object in DB
    const sightPayload = {
      city_id: this.sight.city_id,
      city: this.sight.city,
      latitude: this.sight.latitude,
      longitude: this.sight.longitude,
      name: this.sight[primaryLanguage].name,
      address: this.sight[primaryLanguage].address,
      thumbnail: this.sight.thumbnail,
      watermark_lu: this.sight.watermark_lu,
      watermark_rd: this.sight.watermark_rd,
      left_article: finalLeftArticleId === 0 ? null : finalLeftArticleId,
      preview_media: this.sight.preview_media,
      video_preview: this.sight.video_preview,
    };

    const response = await languageInstance(primaryLanguage).post(
      "/sight",
      sightPayload
    );
    const newSightId = response.data.id; // ID of the newly created sight

    // 4. Update other languages for the sight
    const otherLanguages = (["ru", "en", "zh"] as Language[]).filter(
      (l) => l !== primaryLanguage
    );
    for (const lang of otherLanguages) {
      await languageInstance(lang).patch(`/sight/${newSightId}`, {
        city_id: this.sight.city_id,
        city: this.sight.city,
        latitude: this.sight.latitude,
        longitude: this.sight.longitude,
        name: this.sight[lang].name,
        address: this.sight[lang].address,
        thumbnail: this.sight.thumbnail,
        watermark_lu: this.sight.watermark_lu,
        watermark_rd: this.sight.watermark_rd,
        left_article: finalLeftArticleId === 0 ? null : finalLeftArticleId,
        preview_media: this.sight.preview_media,
        video_preview: this.sight.video_preview,
      });
    }

    // 5. Link Right Articles to the new Sight
    for (let i = 0; i < rightArticleIdsForLink.length; i++) {
      await authInstance.post(`/sight/${newSightId}/article`, {
        article_id: rightArticleIdsForLink[i],
        page_num: i + 1, // Or other logic for page_num
      });
    }

    console.log("Sight created with ID:", newSightId);
    // Optionally: this.clearCreateSight(); // To reset form after successful creation
    this.needLeaveAgree = false;
    return newSightId;
  };

  // --- Media Upload (Generic, used by dialogs) ---
  uploadMedia = async (
    filename: string,
    type: number,
    file: File,
    media_name?: string
  ): Promise<MediaItem> => {
    const formData = new FormData();
    formData.append("file", file);
    formData.append("filename", filename);
    if (media_name) formData.append("media_name", media_name);
    formData.append("type", type.toString());

    try {
      const response = await authInstance.post(`/media`, formData);
      runInAction(() => {
        this.fileToUpload = null;
        this.uploadMediaOpen = false;
      });
      mediaStore.getMedia(); // Refresh global media list
      return {
        id: response.data.id,
        filename: filename, // Or response.data.filename if backend returns it
        media_name: media_name, // Or response.data.media_name
        media_type: type, // Or response.data.type
      };
    } catch (error) {
      console.error("Error uploading media:", error);
      throw error;
    }
  };

  // For Left Article Media
  createLinkWithLeftArticle = async (media: MediaItem) => {
    if (!this.sight.left_article || this.sight.left_article === 10000000) {
      console.warn(
        "Left article not selected or is a placeholder. Cannot link media yet."
      );
      // If it's a placeholder, we could store the media temporarily and link it after the article is created.
      // For simplicity, we'll assume the article must exist.
      // A more robust solution might involve creating the article first if it's a placeholder.
      return;
    }
    try {
      await authInstance.post(`/article/${this.sight.left_article}/media`, {
        media_id: media.id,
        media_order: (this.sight.ru.left.media?.length || 0) + 1,
      });
      runInAction(() => {
        (["ru", "en", "zh"] as Language[]).forEach((lang) => {
          if (!this.sight[lang].left.media) this.sight[lang].left.media = [];
          this.sight[lang].left.media.unshift(media);
        });
      });
    } catch (error) {
      console.error("Error linking media to left article:", error);
      throw error;
    }
  };

  deleteLeftArticleMedia = async (mediaId: string) => {
    if (!this.sight.left_article || this.sight.left_article === 10000000)
      return;
    try {
      await authInstance.delete(`/article/${this.sight.left_article}/media`, {
        data: { media_id: mediaId },
      });
      runInAction(() => {
        (["ru", "en", "zh"] as Language[]).forEach((lang) => {
          if (this.sight[lang].left.media) {
            this.sight[lang].left.media = this.sight[lang].left.media.filter(
              (m) => m.id !== mediaId
            );
          }
        });
      });
    } catch (error) {
      console.error("Error deleting media from left article:", error);
      throw error;
    }
  };

  updateRightArticles = async (articles: any[]) => {
    const articlesIds = articles.map((article) => article.id);

    const sortArticles = (existing: any[]) => {
      const articleMap = new Map(
        existing.map((article) => [article.id, article])
      );
      return articlesIds
        .map((id) => articleMap.get(id))
        .filter(
          (article): article is (typeof existing)[number] =>
            article !== undefined
        );
    };

    this.sight.ru.right = sortArticles(this.sight.ru.right);
    this.sight.en.right = sortArticles(this.sight.en.right);
    this.sight.zh.right = sortArticles(this.sight.zh.right); // теперь zh тоже сортируется одинаково

    this.needLeaveAgree = true;
  };

  needLeaveAgree = false;
  setNeedLeaveAgree = (need: boolean) => {
    this.needLeaveAgree = need;
  };
}

export const createSightStore = new CreateSightStore();
